@@ -1,4 +1,5 @@
 //--------------------------------------------------------------------------------
+
 //
 //	File:		AppleUSBAudioDevice.cpp
 //
@@ -39,7 +40,17 @@ void AppleUSBAudioDevice::free () {
 		mRegisteredEngines->release ();
 		mRegisteredEngines = NULL;
 	}
+	
+	if (mMonoControlsArray)
+	{
+		mMonoControlsArray->release ();
+		mMonoControlsArray = NULL;
+	}
 
+	if (mRetryEQDownloadThread)
+	{
+		thread_call_free (mRetryEQDownloadThread);
+	}
     super::free ();
     debugIOLog ("-AppleUSBAudioDevice[%p]::free ()", this);
 }
@@ -165,17 +176,54 @@ IOReturn AppleUSBAudioDevice::protectedInitHardware (IOService * provider) {
 	err = kIOReturnError;
 	string[0] = 0;
 	stringIndex = mControlInterface->GetInterfaceStringIndex ();
-	if (0 != stringIndex) {
-		err = mControlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
-	} else {
-		stringIndex = mControlInterface->GetDevice()->GetProductStringIndex ();
-		if (0 != stringIndex) {
-			err = mControlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
-		}
+	if (0 != stringIndex) 
+        {
+            err = mControlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
+            // rdar://3886272 - This could fail on some devices, so we should retry
+            if (kIOReturnSuccess != err)
+            {
+                // Try a regular retry once
+                debugIOLog (" ++AppleUSBAudioDevice[%p]::protectedInitHardware - couldn't get string descriptor. Retrying ...", this);
+                err = mControlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
+            }
+            if (kIOReturnSuccess != err)
+            {
+                // Reset the device and try one last time
+                debugIOLog (" ++AppleUSBAudioDevice[%p]::protectedInitHardware - Still couldn't get string descriptor. Resetting device ...", this);
+                mControlInterface->GetDevice()->ResetDevice();      // Doesn't matter if this fails
+				IOSleep (50);										// Give the device 50 ms to get ready
+                debugIOLog (" ++AppleUSBAudioDevice[%p]::protectedInitHardware - Last retry ...", this);
+                err = mControlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
+            }
+	} 
+        else 
+        {
+            stringIndex = mControlInterface->GetDevice()->GetProductStringIndex ();
+            if (0 != stringIndex) 
+            {
+                err = mControlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
+                // rdar://3886272 - This could fail on some devices, so we should retry
+                if (kIOReturnSuccess != err)
+                {
+                    // Try a regular retry once
+                    debugIOLog (" ++AppleUSBAudioDevice[%p]::protectedInitHardware - couldn't get string descriptor. Retrying ...", this);
+                    err = mControlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
+                }
+                if (kIOReturnSuccess != err)
+                {
+                    // Reset the device and try one last time
+                    debugIOLog (" ++AppleUSBAudioDevice[%p]::protectedInitHardware - Still couldn't get string descriptor. Resetting device ...", this);
+                    mControlInterface->GetDevice()->ResetDevice();      // Doesn't matter if this fails
+					IOSleep (50);										// Give the device 50 ms to get ready
+                    debugIOLog (" ++AppleUSBAudioDevice[%p]::protectedInitHardware - Last retry ...", this);
+                    err = mControlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
+                }
+            }
 	}
 
-	if (0 == string[0] || kIOReturnSuccess != err) {
-		strcpy (string, "Unknown USB Audio Device");
+	if (0 == string[0] || kIOReturnSuccess != err) 
+        {
+            strcpy (string, "Unknown USB Audio Device");
 	}
 
 	setDeviceName (string);
@@ -201,6 +249,9 @@ IOReturn AppleUSBAudioDevice::protectedInitHardware (IOService * provider) {
 	FailIf (NULL == mInterfaceLock, Exit);
 
 	resultCode = super::initHardware (provider);
+	
+	// Added for rdar://3993906 . This forces matchPropertyTable () to run again.
+	IOService::registerService();
 
 Exit:
 	debugIOLog ("-AppleUSBAudioDevice[%p]::start (%p)", this, provider);
@@ -378,6 +429,46 @@ IOReturn AppleUSBAudioDevice::message (UInt32 type, IOService * provider, void *
 	return kIOReturnSuccess;
 }
 
+// added for rdar://3959606 . Returns the hub speed (high or full).
+UInt8 AppleUSBAudioDevice::getHubSpeed () {
+	bool					found = false;
+	IOUSBDevice *			usbDevice;
+	const IORegistryPlane *	usbPlane = getPlane (kIOUSBPlane);
+	IORegistryEntry *		currentEntry;
+	UInt8					speed = kUSBDeviceSpeedFull;
+	
+	FailIf (NULL == mControlInterface, Exit);
+	usbDevice = OSDynamicCast (IOUSBDevice, mControlInterface->GetDevice());
+	currentEntry = OSDynamicCast (IORegistryEntry, usbDevice);
+	
+	while (    (!found)
+			&& (currentEntry)
+			&& (usbDevice))
+	{
+		// Searching for a high speed hub. Since it is not possible to run at high speed with a full speed
+		// hub closer to the root hub in the chain, we can stop when we find the first high speed hub.
+		
+		speed = usbDevice->GetSpeed ();
+		if (kUSBDeviceSpeedHigh == speed)
+		{
+			// Must be connected via USB 2.0 hub
+			debugIOLog (" ++AppleUSBAudioDevice::getHubSpeed () = kUSBDeviceSpeedHigh");
+			found = true;
+		}
+		else
+		{
+			// Get parent in USB plane
+			currentEntry = OSDynamicCast (IORegistryEntry, currentEntry->getParentEntry (usbPlane));
+			
+			// If the current registry entry is not a device, this will make usbDevice NULL and exit the loop
+			usbDevice = OSDynamicCast (IOUSBDevice, currentEntry);
+		}
+	} // end while
+	
+Exit:
+	return speed;
+}
+
 SInt32 AppleUSBAudioDevice::getEngineInfoIndex (AppleUSBAudioEngine * inAudioEngine) {
 	OSDictionary *						engineInfo;
 	AppleUSBAudioEngine *				usbAudioEngine;
@@ -423,7 +514,8 @@ IOReturn AppleUSBAudioDevice::doControlStuff (IOAudioEngine *audioEngine, UInt8
 	UInt8								unitID;
 	UInt8								outputTerminalID;
 	Boolean								done;
-	
+
+	debugIOLog ("+AppleUSBAudioDevice::doControlStuff(0x%x, %d, %d)", audioEngine, interfaceNum, altSettingNum);
 	result = kIOReturnError;
 	inputSelector = NULL;
 	done = FALSE;
@@ -475,7 +567,8 @@ IOReturn AppleUSBAudioDevice::doControlStuff (IOAudioEngine *audioEngine, UInt8
 					if (unitID == outputTerminalID) {
 						featureUnitID = getBestFeatureUnitInPath (aPath, kIOAudioControlUsageOutput, interfaceNum, altSettingNum, kVolumeControl);
 						if (featureUnitID) {
-							// Create the input gain controls
+							// Create the output gain controls
+							debugIOLog("----- Creating Output Gain Controls -----");
 							addVolumeControls (usbAudioEngine, featureUnitID, interfaceNum, altSettingNum, kIOAudioControlUsageOutput);
 							featureUnitID = 0;
 						}
@@ -539,6 +632,7 @@ IOReturn AppleUSBAudioDevice::doControlStuff (IOAudioEngine *audioEngine, UInt8
 									featureUnitID = getBestFeatureUnitInPath (aPath, kIOAudioControlUsageInput, interfaceNum, altSettingNum, kVolumeControl);
 									if (featureUnitID) {
 										// Create the input gain controls
+										debugIOLog("----- Creating Intput Gain Controls -----");
 										addVolumeControls (usbAudioEngine, featureUnitID, interfaceNum, altSettingNum, kIOAudioControlUsageInput);
 										featureUnitID = 0;
 									}
@@ -582,6 +676,7 @@ IOReturn AppleUSBAudioDevice::doControlStuff (IOAudioEngine *audioEngine, UInt8
 	result = kIOReturnSuccess;
 
 Exit:
+	debugIOLog ("-AppleUSBAudioDevice::doControlStuff(0x%x, %d, %d)", audioEngine, interfaceNum, altSettingNum);
 	return result;
 }
 
@@ -828,11 +923,12 @@ void AppleUSBAudioDevice::addVolumeControls (AppleUSBAudioEngine * usbAudioEngin
 	SInt16								deviceCur;
 	SInt16								deviceMin;
 	SInt16								deviceMax;
-	SInt16								volRes;
+	UInt16								volRes;
 	SInt16								offset;
 	UInt8								channelNum;
 	UInt8								controlInterfaceNum;
 
+	debugIOLog ("+AppleUSBAudioDevice::addVolumeControls (0x%x, %d, %d, %d, %u)", usbAudioEngine, featureUnitID, interfaceNum, altSettingNum, usage);
 	engineInfoIndex = getEngineInfoIndex (usbAudioEngine);
 	FailIf (-1 == engineInfoIndex, Exit);
 	FailIf (NULL == mControlInterface, Exit);
@@ -842,19 +938,30 @@ void AppleUSBAudioDevice::addVolumeControls (AppleUSBAudioEngine * usbAudioEngin
 	inputGainControlsArray = NULL;
 	passThruVolControlsArray = NULL;
 	outputVolControlsArray = NULL;
+	
+	// remove mono controls array if adding volume controls for output
+	if (    (kIOAudioControlUsageOutput == usage)
+	     && (NULL != mMonoControlsArray))
+	{
+		mMonoControlsArray->release ();
+		mMonoControlsArray = NULL;
+	}
 
 	controlInterfaceNum = mControlInterface->GetInterfaceNumber ();
 	for (channelNum = 0; channelNum <= mUSBAudioConfig->GetNumControls (controlInterfaceNum, 0, featureUnitID); channelNum++) {
-		debugIOLog ("checking channel %d for vol controls", channelNum);
+		debugIOLog ("Checking channel %d for volume controls", channelNum);
 		if (mUSBAudioConfig->ChannelHasVolumeControl (controlInterfaceNum, 0, featureUnitID, channelNum)) {
-			debugIOLog ("creating vol controls for channel %d", channelNum);
-			deviceCur = getCurVolume (featureUnitID, channelNum);
-			deviceMin = getMinVolume (featureUnitID, channelNum);
+			debugIOLog ("Creating volume controls for channel %d", channelNum);
+			FailIf (kIOReturnSuccess != getCurVolume (featureUnitID, channelNum, &deviceCur), Error);
+			debugIOLog ("deviceCur = %x", deviceCur);
+			FailIf (kIOReturnSuccess != getMinVolume (featureUnitID, channelNum, &deviceMin), Error);
 			debugIOLog ("deviceMin = %x", deviceMin);
-			deviceMax = getMaxVolume (featureUnitID, channelNum);
+			FailIf (kIOReturnSuccess != getMaxVolume (featureUnitID, channelNum, &deviceMax), Error);
 			debugIOLog ("deviceMax = %x", deviceMax);
-			volRes = getVolumeResolution (featureUnitID, channelNum);
+			getVolumeResolution (featureUnitID, channelNum, &volRes);
 			debugIOLog ("volRes = %x", volRes);
+			debugIOLog ("Attempting to set volume to current volume ...");
+			FailIf (kIOReturnSuccess != setCurVolume (featureUnitID, channelNum, HostToUSBWord(deviceCur)), Error);
 
 			if ((SInt16)0x8000 == deviceMin) {
 				deviceMin = (SInt16)0x8001;
@@ -896,10 +1003,34 @@ void AppleUSBAudioDevice::addVolumeControls (AppleUSBAudioEngine * usbAudioEngin
 					} else {
 						outputVolControlsArray->setObject (theLevelControl);
 					}
+					
+					// add channel number to mono output controls array if necessary
+					if (mDeviceIsInMonoMode)
+					{
+						OSNumber *number = OSNumber::withNumber (channelNum, 8);
+						if (NULL == mMonoControlsArray)
+						{
+							mMonoControlsArray = OSArray::withObjects ((const OSObject **) &number, 1);
+						}
+						else
+						{
+							mMonoControlsArray->setObject (number);
+						}
+						debugIOLog ("addVolumeControls: added channel %d to mono controls array", channelNum);
+						number->release ();
+					}
+					
 					break;
 			}
 			theLevelControl->release ();
+		} else {
+			debugIOLog ("Channel %d has no volume controls; skipping ...", channelNum);
 		}
+		goto NoError;
+Error: 
+		debugIOLog ("Error creating controls for channel %d!", channelNum);
+NoError:
+		debugIOLog ("Done with channel %d", channelNum);
 	}
 
 	if (NULL != inputGainControlsArray) {
@@ -916,6 +1047,7 @@ void AppleUSBAudioDevice::addVolumeControls (AppleUSBAudioEngine * usbAudioEngin
 	}
 
 Exit:
+	debugIOLog ("-AppleUSBAudioDevice::addVolumeControls (0x%x, %d, %d, %d, %u)", usbAudioEngine, featureUnitID, interfaceNum, altSettingNum, usage);
 	return;
 }
 
@@ -927,6 +1059,7 @@ void AppleUSBAudioDevice::addMuteControl (AppleUSBAudioEngine * usbAudioEngine,
 	SInt16								deviceCur;
 	UInt8								channelNum;
 	UInt8								controlInterfaceNum;
+	IOReturn							resultCode;
 
 	engineInfoIndex = getEngineInfoIndex (usbAudioEngine);
 	FailIf (-1 == engineInfoIndex, Exit);
@@ -939,10 +1072,10 @@ void AppleUSBAudioDevice::addMuteControl (AppleUSBAudioEngine * usbAudioEngine,
 	controlInterfaceNum = mControlInterface->GetInterfaceNumber ();
 	for (channelNum = 0; channelNum <= mUSBAudioConfig->GetNumControls (controlInterfaceNum, 0, featureUnitID); channelNum++) {
 		if (mUSBAudioConfig->ChannelHasMuteControl (controlInterfaceNum, 0, featureUnitID, channelNum)) {
-			deviceCur = getCurMute (featureUnitID, channelNum);
+			resultCode = getCurMute (featureUnitID, channelNum, &deviceCur);
 
 			theMuteControl = IOAudioToggleControl::createMuteControl (deviceCur, channelNum, 0, featureUnitID, usage);
-			setCurMute (featureUnitID, channelNum, deviceCur);
+			setCurMute (featureUnitID, channelNum, HostToUSBWord(deviceCur));
 			FailIf (NULL == theMuteControl, Exit);
 			theMuteControl->setValueChangeHandler (controlChangedHandler, this);
 			usbAudioEngine->addDefaultAudioControl (theMuteControl);
@@ -990,14 +1123,17 @@ IOFixed AppleUSBAudioDevice::ConvertUSBVolumeTodB (SInt16 volume) {
 	return dBVolumeFixed;
 }
 
-SInt16 AppleUSBAudioDevice::getFeatureUnitSetting (UInt8 controlSelector, UInt8 unitID, UInt8 channelNumber, UInt8 requestType) {
-    IOUSBDevRequestDesc					devReq;
-	IOReturn							result;
+IOReturn AppleUSBAudioDevice::getFeatureUnitSetting (UInt8 controlSelector, UInt8 unitID, UInt8 channelNumber, UInt8 requestType, SInt16 * target) {
+    IOReturn							result;
+	IOUSBDevRequestDesc					devReq;
 	UInt16								theSetting;
 	IOBufferMemoryDescriptor *			theSettingDesc = NULL;
 	UInt8								length;
 
+	result = kIOReturnError;
+	// Initialize theSetting so that 
 	theSetting = 0;
+	FailIf (NULL == target, Exit);
 	FailIf (NULL == mControlInterface, Exit);
 
 	switch (controlSelector) {
@@ -1023,12 +1159,15 @@ SInt16 AppleUSBAudioDevice::getFeatureUnitSetting (UInt8 controlSelector, UInt8
 	result = deviceRequest (&devReq);
 	FailIf (kIOReturnSuccess != result, Exit);
 	memcpy (&theSetting, theSettingDesc->getBytesNoCopy (), length);
-
+	
 Exit:
 	if (NULL != theSettingDesc) {
 		theSettingDesc->release ();
 	}
-	return USBToHostWord (theSetting);
+	if (NULL != target) {
+		*target = USBToHostWord (theSetting);
+	}
+	return result;
 }
 
 IOReturn AppleUSBAudioDevice::setFeatureUnitSetting (UInt8 controlSelector, UInt8 unitID, UInt8 channelNumber, UInt8 requestType, UInt16 newValue, UInt16 newValueLen) {
@@ -1060,28 +1199,28 @@ Exit:
 	return result;
 	
 DeviceInactive:
-	debugIOLog("AppleUSBAudioLevelControl::SetCurVolume ERROR attempt to send a device request to and inactive device");
+	debugIOLog("AppleUSBAudioLevelControl::SetCurVolume ERROR attempt to send a device request to an inactive device");
 	goto Exit;
 }
 
-SInt16 AppleUSBAudioDevice::getCurMute (UInt8 unitID, UInt8 channelNumber) {
-	return getFeatureUnitSetting (MUTE_CONTROL, unitID, channelNumber, GET_CUR);
+IOReturn AppleUSBAudioDevice::getCurMute (UInt8 unitID, UInt8 channelNumber, SInt16 * target) {
+	return getFeatureUnitSetting (MUTE_CONTROL, unitID, channelNumber, GET_CUR, target);
 }
 
-SInt16 AppleUSBAudioDevice::getCurVolume (UInt8 unitID, UInt8 channelNumber) {
-	return getFeatureUnitSetting (VOLUME_CONTROL, unitID, channelNumber, GET_CUR);
+IOReturn AppleUSBAudioDevice::getCurVolume (UInt8 unitID, UInt8 channelNumber, SInt16 * target) {
+	return getFeatureUnitSetting (VOLUME_CONTROL, unitID, channelNumber, GET_CUR, target);
 }
 
-SInt16 AppleUSBAudioDevice::getMaxVolume (UInt8 unitID, UInt8 channelNumber) {
-	return getFeatureUnitSetting (VOLUME_CONTROL, unitID, channelNumber, GET_MAX);
+IOReturn AppleUSBAudioDevice::getMaxVolume (UInt8 unitID, UInt8 channelNumber, SInt16 * target) {
+	return getFeatureUnitSetting (VOLUME_CONTROL, unitID, channelNumber, GET_MAX, target);
 }
 
-SInt16 AppleUSBAudioDevice::getMinVolume (UInt8 unitID, UInt8 channelNumber) {
-	return getFeatureUnitSetting (VOLUME_CONTROL, unitID, channelNumber, GET_MIN);
+IOReturn AppleUSBAudioDevice::getMinVolume (UInt8 unitID, UInt8 channelNumber, SInt16 * target) {
+	return getFeatureUnitSetting (VOLUME_CONTROL, unitID, channelNumber, GET_MIN, target);
 }
 
-UInt16 AppleUSBAudioDevice::getVolumeResolution (UInt8 unitID, UInt8 channelNumber) {
-	return getFeatureUnitSetting (VOLUME_CONTROL, unitID, channelNumber, GET_RES);
+IOReturn AppleUSBAudioDevice::getVolumeResolution (UInt8 unitID, UInt8 channelNumber, UInt16 * target) {
+	return getFeatureUnitSetting (VOLUME_CONTROL, unitID, channelNumber, GET_RES, (SInt16 *) target);
 }
 
 IOReturn AppleUSBAudioDevice::setCurVolume (UInt8 unitID, UInt8 channelNumber, SInt16 volume) {
@@ -1148,28 +1287,76 @@ IOReturn AppleUSBAudioDevice::doVolumeControlChange (IOAudioControl * audioContr
 	SInt16								newVolume;
 	SInt16								deviceMin;
 	SInt16								offset;
+	UInt16								volRes;
 	UInt8								unitID;
 	UInt8								channelNum;
 
+	
 	unitID = audioControl->getControlID ();
 	channelNum = audioControl->getChannelID ();
+	result = kIOReturnError;
 
-	deviceMin = getMinVolume (unitID, channelNum);
-	offset = -deviceMin;
+	if (    (kIOAudioControlUsageInput == audioControl->getUsage())
+	     || (FALSE == mDeviceIsInMonoMode))
+	{
+		getMinVolume (unitID, channelNum, &deviceMin);
+		offset = -deviceMin;
 
-	if (newValue < 0) {
-		newVolume = 0x8000;
-	} else {
-		if (newValue > 0) {
-			newVolume = ((newValue - 1) * getVolumeResolution (unitID, channelNum)) - offset;
-		} else {
-			newVolume = (newValue * getVolumeResolution (unitID, channelNum)) - offset;
+		if (newValue < 0) 
+		{
+			newVolume = 0x8000;
+		} 
+		else 
+		{
+			getVolumeResolution (unitID, channelNum, &volRes);
+			if (newValue > 0) 
+			{
+				newVolume = ((newValue - 1) * volRes) - offset;
+			} 
+			else 
+			{
+				newVolume = (newValue * volRes) - offset;
+			}
+		}
+
+		debugIOLog ("doVolumeControlChange: setting volume to 0x%x", newVolume);
+		result = setCurVolume (unitID, channelNum, HostToUSBWord (newVolume));
+	
+	}
+	else
+	{
+		UInt8 i;
+		FailIf (NULL == mMonoControlsArray, Exit);
+		debugIOLog ("doVolumeControlChange: performing mono volume control change");
+		for (i = 0; i < mMonoControlsArray->getCount (); i++)
+		{
+			channelNum = ((OSNumber *) mMonoControlsArray->getObject(i))->unsigned8BitValue ();
+			getMinVolume (unitID, channelNum, &deviceMin);
+			offset = -deviceMin;
+		
+			if (newValue < 0) 
+			{
+				newVolume = 0x8000;
+			} 
+			else 
+			{
+				getVolumeResolution (unitID, channelNum, &volRes);
+				if (newValue > 0) 
+				{
+					newVolume = ((newValue - 1) * volRes) - offset;
+				} 
+				else 
+				{
+					newVolume = (newValue * volRes) - offset;
+				}
+			}
+		
+			result = setCurVolume (unitID, channelNum, HostToUSBWord (newVolume));
+			debugIOLog ("doVolumeControlChange: set volume for channel %d to 0x%x = %d", channelNum, newVolume, result);
 		}
 	}
-
-	debugIOLog ("setting volume to 0x%x", newVolume);
-	result = setCurVolume (unitID, channelNum, HostToUSBWord (newVolume));
-
+	
+Exit:
 	return result;
 }
 
@@ -1487,6 +1674,11 @@ Exit:
 	return result;
 }
 
+void AppleUSBAudioDevice::setMonoState (Boolean state)
+{
+	mDeviceIsInMonoMode = state;
+}
+
 IOReturn AppleUSBAudioDevice::createControlsForInterface (IOAudioEngine *audioEngine, UInt8 interfaceNum, UInt8 altSettingNum) {
     AppleUSBAudioEngine *				usbAudioEngine;
     IOReturn							result;
@@ -1553,7 +1745,8 @@ OSArray * AppleUSBAudioDevice::BuildConnectionGraph (UInt8 controlInterfaceNum)
 		}
 	}
 #endif
-
+	pathsFromOutputTerminalN->release();
+	
 Exit:
 	return allOutputTerminalPaths;
 }
@@ -1926,7 +2119,6 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequestDesc * request, IOUS
 	UInt32							timeout;
 	Boolean							done;
 
-	debugIOLog ("+AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", this, request, completion);
 	result = kIOReturnSuccess;
 	FailIf (NULL == mInterfaceLock, Exit);
 	IORecursiveLockLock (mInterfaceLock);
@@ -1946,7 +2138,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequestDesc * request, IOUS
     }
 	IORecursiveLockUnlock (mInterfaceLock);
 
-	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", this, request, completion, result);
+	debugIOLog ("++AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", this, request, completion, result);
 
 Exit:
 	return result;
@@ -1957,7 +2149,6 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest * request, IOUSBCom
 	UInt32							timeout;
 	Boolean							done;
 
-	debugIOLog ("+AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", this, request, completion);
 	result = kIOReturnSuccess;
 	FailIf (NULL == mInterfaceLock, Exit);
 	IORecursiveLockLock (mInterfaceLock);
@@ -1977,7 +2168,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest * request, IOUSBCom
     }
 	IORecursiveLockUnlock (mInterfaceLock);
 
-	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", this, request, completion, result);
+	debugIOLog ("++AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", this, request, completion, result);
 
 Exit:
 	return result;
@@ -1988,7 +2179,6 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest *request, AppleUSBA
 	UInt32							timeout;
 	Boolean							done;
 
-	debugIOLog ("+AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", self, request, completion);
 	result = kIOReturnSuccess;
 	FailIf (NULL == self->mInterfaceLock, Exit);
 	IORecursiveLockLock (self->mInterfaceLock);
@@ -2008,7 +2198,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest *request, AppleUSBA
 	}
 	IORecursiveLockUnlock (self->mInterfaceLock);
 
-	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", self, request, completion, result);
+	debugIOLog ("++AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", self, request, completion, result);
 
 Exit:
 	return result;
@@ -2045,3 +2235,41 @@ void AppleUSBAudioDevice::release() const
 }
 
 #endif
+
+// Implemented for rdar://3993906 . Allows matching based on a custom dictionary.
+bool AppleUSBAudioDevice::matchPropertyTable(OSDictionary * table, SInt32 *score)
+{
+	bool		returnValue = false;
+	OSObject *	deviceName;
+	
+	// debugIOLog ("+AppleUSBAudioDevice[%p]::matchPropertyTable (%p, %p)", this, table, score);
+	deviceName = table->getObject(kIOAudioDeviceNameKey);
+	if (deviceName)
+	{
+		// This custom dictionary wants the device to have a name.
+		if (getProperty (kIOAudioDeviceNameKey))
+		{
+			// No need to match name; it exists, and that's all that matters
+			returnValue = true;
+		}
+		else
+		{
+			// Device doesn't have a name yet, so don't match
+			returnValue = false;
+		}
+	}
+	else
+	{
+		// This is our standard matchPropertyTable implementation
+		returnValue = super::matchPropertyTable (table, score);
+	}
+	
+	if (    (deviceName)
+		 && (returnValue))
+	{
+		debugIOLog ("++AppleUSBAudioDevice[%p]::matchPropertyTable (%p, %p) = %d (custom dictionary match)", 
+					this, table, score, returnValue);
+	}
+	
+	return returnValue;
+}
