@@ -87,7 +87,9 @@ protected:
     IORecursiveLock *					mInterfaceLock;
 	Boolean								mTerminatingDriver;
 	thread_call_t						mInitHardwareThread;
-
+	thread_call_t						mRetryEQDownloadThread;
+	Boolean								mDeviceIsInMonoMode;
+	OSArray *							mMonoControlsArray;		// this flag is set by AppleUSBAudioEngine::performFormatChange
 	OSArray *							mRegisteredEngines;
 
 public:
@@ -125,17 +127,17 @@ public:
 	virtual	IOReturn		doSelectorControlChange (IOAudioControl * audioControl, SInt32 oldValue, SInt32 newValue);
 	virtual	UInt8			getSelectorSetting (UInt8 selectorID);
 	virtual	IOReturn		setSelectorSetting (UInt8 selectorID, UInt8 setting);
-	virtual	SInt16			getFeatureUnitSetting (UInt8 controlSelector, UInt8 unitID, UInt8 channelNumber, UInt8 requestType);
+	virtual	IOReturn		getFeatureUnitSetting (UInt8 controlSelector, UInt8 unitID, UInt8 channelNumber, UInt8 requestType, SInt16 * target);
 	virtual	IOReturn		setFeatureUnitSetting (UInt8 controlSelector, UInt8 unitID, UInt8 channelNumber, UInt8 requestType, UInt16 newValue, UInt16 newValueLen);
 	virtual	OSArray *		getPlaythroughPaths ();
 	virtual	UInt8			getBestFeatureUnitInPath (OSArray * thePath, UInt32 direction, UInt8 interfaceNum, UInt8 altSettingNum, UInt32 controlTypeWanted);
 	virtual	void			addVolumeControls (AppleUSBAudioEngine * usbAudioEngine, UInt8 featureUnitID, UInt8 interfaceNum, UInt8 altSettingNum, UInt32 usage);
 	virtual	void			addMuteControl (AppleUSBAudioEngine * usbAudioEngine, UInt8 featureUnitID, UInt8 interfaceNum, UInt8 altSettingNum, UInt32 usage);
-	virtual	SInt16			getCurMute (UInt8 unitID, UInt8 channelNumber);
-	virtual	SInt16			getCurVolume (UInt8 unitID, UInt8 channelNumber);
-	virtual	SInt16			getMaxVolume (UInt8 unitID, UInt8 channelNumber);
-	virtual	SInt16			getMinVolume (UInt8 unitID, UInt8 channelNumber);
-	virtual	UInt16			getVolumeResolution (UInt8 unitID, UInt8 channelNumber);
+	virtual	IOReturn		getCurMute (UInt8 unitID, UInt8 channelNumber, SInt16 * target);
+	virtual	IOReturn		getCurVolume (UInt8 unitID, UInt8 channelNumber, SInt16 * target);
+	virtual	IOReturn		getMaxVolume (UInt8 unitID, UInt8 channelNumber, SInt16 * target);
+	virtual	IOReturn		getMinVolume (UInt8 unitID, UInt8 channelNumber, SInt16 * target);
+	virtual	IOReturn		getVolumeResolution (UInt8 unitID, UInt8 channelNumber, UInt16 * target);
 	virtual	IOReturn		setCurVolume (UInt8 unitID, UInt8 channelNumber, SInt16 volume);
 	virtual	IOReturn		setCurMute (UInt8 unitID, UInt8 channelNumber, SInt16 mute);
 	virtual	IOReturn		doInputSelectorChange (IOAudioControl *audioControl, SInt32 oldValue, SInt32 newValue);
@@ -143,6 +145,8 @@ public:
 	virtual	IOReturn		doToggleControlChange (IOAudioControl *audioControl, SInt32 oldValue, SInt32 newValue);
 	virtual IOReturn		doPassThruSelectorChange (IOAudioControl * audioControl, SInt32 oldValue, SInt32 newValue);
 	virtual	IOFixed			ConvertUSBVolumeTodB (SInt16 volume);
+	virtual void			setMonoState (Boolean state);
+	virtual	UInt8			getHubSpeed (); // added for rdar://3959606
 
 	virtual	IOReturn		deviceRequest (IOUSBDevRequest * request, IOUSBCompletion * completion = NULL);			// Depricated, don't use
 	virtual	IOReturn		deviceRequest (IOUSBDevRequestDesc * request, IOUSBCompletion * completion = NULL);
@@ -152,6 +156,7 @@ public:
     virtual void			retain() const;
     virtual void			release() const;
 #endif
+	virtual bool			matchPropertyTable (OSDictionary * table, SInt32 *score);
 };
 
 #endif /* _APPLEUSBAUDIODEVICE_H */
