@@ -80,16 +80,16 @@ bool AppleUSBAudioLevelControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
 			break;
 	}
 
-	currentValue = GetCurVolume (interfaceNumber, channelNumber, &ret);
+	ret = GetCurVolume (interfaceNumber, channelNumber, &currentValue);
 	FailIf (kIOReturnSuccess != ret, Exit);
 	debugIOLog ("channelNumber %d, currentValue = 0x%X, ", channelNumber, currentValue);
-	volRes = GetVolumeResolution (interfaceNumber, channelNumber, &ret);
+	ret = GetVolumeResolution (interfaceNumber, channelNumber, &volRes);
 	FailIf (kIOReturnSuccess != ret, Exit);
 	debugIOLog ("vol res = %d, ", volRes);
-	deviceMin = GetMinVolume (interfaceNumber, channelNumber, &ret);
+	ret = GetMinVolume (interfaceNumber, channelNumber, &deviceMin);
 	FailIf (kIOReturnSuccess != ret, Exit);
 	debugIOLog ("deviceMin = 0x%X, ", deviceMin);
-	deviceMax = GetMaxVolume (interfaceNumber, channelNumber, &ret);
+	ret = GetMaxVolume (interfaceNumber, channelNumber, &deviceMax);
 	FailIf (kIOReturnSuccess != ret, Exit);
 	debugIOLog ("deviceMax = 0x%X", deviceMax);
 
@@ -268,10 +268,15 @@ void AppleUSBAudioLevelControl::updateUSBValue (SInt32 newValue) {
     debugIOLog ("-AppleUSBAudioLevelControl[%p]::updateUSBValue (%d)", this, newValue);
 }
 
-SInt16 AppleUSBAudioLevelControl::GetCurVolume (UInt8 interfaceNumber, UInt8 channelNumber, IOReturn * error) {
+IOReturn AppleUSBAudioLevelControl::GetCurVolume (UInt8 interfaceNumber, UInt8 channelNumber, SInt16 * target) {
     IOUSBDevRequest				devReq;
     SInt16						theVolume;
-
+	IOReturn					result;
+	
+	result = kIOReturnError;
+	theVolume = 0;
+	FailIf ( NULL == target, Exit );
+	
     devReq.bmRequestType = USBmakebmRequestType (kUSBIn, kUSBClass, kUSBInterface);
     devReq.bRequest = GET_CUR;
     devReq.wValue = (controlSelector << 8) | channelNumber;
@@ -279,20 +284,25 @@ SInt16 AppleUSBAudioLevelControl::GetCurVolume (UInt8 interfaceNumber, UInt8 cha
     devReq.wLength = 2;
     devReq.pData = &theVolume;
 
-	*error = usbDeviceRequest (&devReq, callerRefCon);
-    FailIf (kIOReturnSuccess != *error, Error);
+	result = usbDeviceRequest (&devReq, callerRefCon);
+    FailIf (kIOReturnSuccess != result, Exit);
 
 Exit:
-    return USBToHostWord (theVolume);
-Error:
-	theVolume = 0;
-	goto Exit;
+	if (NULL != target) {
+		* target = USBToHostWord (theVolume);
+		}
+    return result;
 }
 
-SInt16 AppleUSBAudioLevelControl::GetMaxVolume (UInt8 interfaceNumber, UInt8 channelNumber, IOReturn * error) {
-    IOUSBDevRequest				devReq;
+IOReturn AppleUSBAudioLevelControl::GetMaxVolume (UInt8 interfaceNumber, UInt8 channelNumber, SInt16 * target) {
+    IOReturn					result;
+	IOUSBDevRequest				devReq;
     SInt16						theVolume;
 
+	result = kIOReturnError;
+	theVolume = 0;
+	FailIf (NULL == target, Exit);
+	
     devReq.bmRequestType = USBmakebmRequestType (kUSBIn, kUSBClass, kUSBInterface);
     devReq.bRequest = GET_MAX;
     devReq.wValue = (controlSelector << 8) | channelNumber;
@@ -300,41 +310,51 @@ SInt16 AppleUSBAudioLevelControl::GetMaxVolume (UInt8 interfaceNumber, UInt8 cha
     devReq.wLength = 2;
     devReq.pData = &theVolume;
 
-	*error = usbDeviceRequest (&devReq, callerRefCon);
-    FailIf (kIOReturnSuccess != *error, Error);
+	result = usbDeviceRequest (&devReq, callerRefCon);
+    FailIf (kIOReturnSuccess != result, Exit);
 
 Exit:
-    return USBToHostWord (theVolume);
-Error:
-	theVolume = 0;
-	goto Exit;
+	if (NULL != target) {
+		* target = USBToHostWord (theVolume);
+	}
+    return result;
 }
 
-SInt16 AppleUSBAudioLevelControl::GetMinVolume (UInt8 interfaceNumber, UInt8 channelNumber, IOReturn * error) {
-    IOUSBDevRequest				devReq;
+IOReturn AppleUSBAudioLevelControl::GetMinVolume (UInt8 interfaceNumber, UInt8 channelNumber, SInt16 * target) {
+    IOReturn					result;
+	IOUSBDevRequest				devReq;
     SInt16						theVolume;
 
-    devReq.bmRequestType = USBmakebmRequestType (kUSBIn, kUSBClass, kUSBInterface);
+    result = kIOReturnError;
+	theVolume = 0;
+	FailIf (NULL == target, Exit);
+	
+	devReq.bmRequestType = USBmakebmRequestType (kUSBIn, kUSBClass, kUSBInterface);
     devReq.bRequest = GET_MIN;
     devReq.wValue = (controlSelector << 8) | channelNumber;
     devReq.wIndex = (unitID << 8) | interfaceNumber;
     devReq.wLength = 2;
     devReq.pData = &theVolume;
 
-	*error = usbDeviceRequest (&devReq, callerRefCon);
-    FailIf (kIOReturnSuccess != *error, Error);
+	result = usbDeviceRequest (&devReq, callerRefCon);
+    FailIf (kIOReturnSuccess != result, Exit);
 
 Exit:
-    return USBToHostWord (theVolume);
-Error:
-	theVolume = 0;
-	goto Exit;
+	if (NULL != target) {
+		* target = USBToHostWord (theVolume);
+	}
+    return result;
 }
 
-UInt16 AppleUSBAudioLevelControl::GetVolumeResolution (UInt8 interfaceNumber, UInt8 channelNumber, IOReturn * error) {
-    IOUSBDevRequest				devReq;
+IOReturn AppleUSBAudioLevelControl::GetVolumeResolution (UInt8 interfaceNumber, UInt8 channelNumber, UInt16 * target) {
+    IOReturn					result;
+	IOUSBDevRequest				devReq;
     UInt16						theResolution;
 
+	result = kIOReturnError;
+	theResolution = 0;
+	FailIf (NULL == target, Exit);
+	
     devReq.bmRequestType = USBmakebmRequestType (kUSBIn, kUSBClass, kUSBInterface);
     devReq.bRequest = GET_RES;
     devReq.wValue = (controlSelector << 8) | channelNumber;
@@ -342,14 +362,14 @@ UInt16 AppleUSBAudioLevelControl::GetVolumeResolution (UInt8 interfaceNumber, UI
     devReq.wLength = 2;
     devReq.pData = &theResolution;
 
-	*error = usbDeviceRequest (&devReq, callerRefCon);
-    FailIf (kIOReturnSuccess != *error, Error);
+	result = usbDeviceRequest (&devReq, callerRefCon);
+    FailIf (kIOReturnSuccess != result, Exit);
 
 Exit:
-    return USBToHostWord (theResolution);
-Error:
-	theResolution = 0;
-	goto Exit;
+	if (NULL != target) {
+		* target = USBToHostWord (theResolution);
+	}
+    return result;
 }
 
 IOReturn AppleUSBAudioLevelControl::SetCurVolume (UInt8 interfaceNumber, UInt8 channelNumber, SInt16 volume) {
