@@ -6,7 +6,7 @@ OSDefineMetaClassAndStructors(AppleUSBAudioMuteControl, IOAudioToggleControl)
 AppleUSBAudioMuteControl *AppleUSBAudioMuteControl::create (UInt8 theUnitID, UInt8 theInterfaceNumber, UInt8 theChannelNumber, USBDeviceRequest theUSBDeviceRequest, void *theCallerRefCon, UInt32 usage, UInt32 subType, UInt32 controlID) {
     AppleUSBAudioMuteControl 			*control;
 
-    debug7IOLog ("+AppleUSBAudioMuteControl::create (%d, %d, %d, 0x%x, %lX, %lX)\n", theUnitID, theInterfaceNumber, theChannelNumber, theUSBDeviceRequest, usage, controlID);
+    debugIOLog ("+AppleUSBAudioMuteControl::create (%d, %d, %d, 0x%x, %lX, %lX)", theUnitID, theInterfaceNumber, theChannelNumber, theUSBDeviceRequest, usage, controlID);
     control = new AppleUSBAudioMuteControl;
 
     if (control) {
@@ -16,7 +16,7 @@ AppleUSBAudioMuteControl *AppleUSBAudioMuteControl::create (UInt8 theUnitID, UIn
         }
     }
 
-    debug7IOLog ("-AppleUSBAudioMuteControl[0x%x]::create (%d, %d, %d, 0x%x, %lX, %lX)\n", theUnitID, theInterfaceNumber, theChannelNumber, theUSBDeviceRequest, usage, controlID);
+    debugIOLog ("-AppleUSBAudioMuteControl[0x%x]::create (%d, %d, %d, 0x%x, %lX, %lX)", theUnitID, theInterfaceNumber, theChannelNumber, theUSBDeviceRequest, usage, controlID);
     return control;
 }
 
@@ -25,8 +25,9 @@ bool AppleUSBAudioMuteControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
     UInt8								currentValue;
     IOReturn							ret;
     Boolean								result;
+    UInt8								theValue;
     
-    debug8IOLog ("+AppleUSBAudioMuteControl[0x%x]::init (%d, %d, %d, 0x%x, 0x%x, %lX)\n", this, theUnitID, theInterfaceNumber, theChannelNumber, theUSBDeviceRequest, usage, properties);
+    debugIOLog ("+AppleUSBAudioMuteControl[0x%x]::init (%d, %d, %d, 0x%x, 0x%x, %lX)", this, theUnitID, theInterfaceNumber, theChannelNumber, theUSBDeviceRequest, usage, properties);
     result = FALSE;
     FailIf (NULL == theUSBDeviceRequest, Exit);
 
@@ -50,12 +51,17 @@ bool AppleUSBAudioMuteControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
             channelName = kIOAudioControlChannelNameRight;
             break;
         case 0xff:
-            debugIOLog ("++AppleUSBAudioMuteControl: Does not support channel number 0xff.\n");
+            debugIOLog ("++AppleUSBAudioMuteControl: Does not support channel number 0xff.");
             return FALSE;
         default:
             channelName = "Unknown";
             break;
     }
+	
+	if (kIOAudioToggleControlSubTypeLFEMute == subType) {
+		theValue = 0; // 0 = un-mute
+		ret = SetCurMute (interfaceNumber, channelNumber, theValue);
+	}
 
 	currentValue = GetCurMute (interfaceNumber, channelNumber, &ret);
     FailIf (kIOReturnSuccess != ret, Exit);
@@ -64,19 +70,19 @@ bool AppleUSBAudioMuteControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
     result = TRUE;
     
 Exit:
-    debug8IOLog ("-AppleUSBAudioMuteControl[0x%x]::init (%d, %d, %d, 0x%x, 0x%x, %lX)\n", this, theUnitID, theInterfaceNumber, theChannelNumber, theUSBDeviceRequest, usage, properties);
+    debugIOLog ("-AppleUSBAudioMuteControl[0x%x]::init (%d, %d, %d, 0x%x, 0x%x, %lX)", this, theUnitID, theInterfaceNumber, theChannelNumber, theUSBDeviceRequest, usage, properties);
     return result;
 }
 
 void AppleUSBAudioMuteControl::free () {
-    debug2IOLog ("+AppleUSBAudioMuteControl[0x%x]::free ()\n", this);
+    debugIOLog ("+AppleUSBAudioMuteControl[0x%x]::free ()", this);
 
     if (setValueThreadCall) {
         thread_call_free (setValueThreadCall);
         setValueThreadCall = NULL;
     }
 
-    debug2IOLog ("-AppleUSBAudioMuteControl[0x%x]::free ()\n", this);
+    debugIOLog ("-AppleUSBAudioMuteControl[0x%x]::free ()", this);
     super::free ();
 }
 
@@ -84,12 +90,12 @@ IOReturn AppleUSBAudioMuteControl::performValueChange (OSObject * newValue) {
 	OSNumber *					newValueAsNumber;
 	SInt32						newValueAsSInt32;
 
-    debug3IOLog ("+AppleUSBAudioMuteControl[0x%x]::performValueChange (%d)\n", this, newValue); 
+    debugIOLog ("+AppleUSBAudioMuteControl[0x%x]::performValueChange (%d)", this, newValue); 
 
 	newValueAsNumber = OSDynamicCast (OSNumber, newValue);
 	FailIf (NULL == newValueAsNumber, Exit);
 	newValueAsSInt32 = newValueAsNumber->unsigned32BitValue ();
-	debug3IOLog ("++AppleUSBAudioMuteControl[0x%x]::performValueChange (%ld)\n", this, newValueAsSInt32);
+	debugIOLog ("++AppleUSBAudioMuteControl[0x%x]::performValueChange (%ld)", this, newValueAsSInt32);
 
     // updateUSBValue ();
     // We should just be able to make an asynchronous deviceRequest, but for some reason that doesn't want to work
@@ -98,7 +104,7 @@ IOReturn AppleUSBAudioMuteControl::performValueChange (OSObject * newValue) {
     assert (setValueThreadCall);
     thread_call_enter1 (setValueThreadCall, (thread_call_param_t)newValueAsSInt32);
     
-    debug2IOLog ("-AppleUSBAudioMuteControl::performValueChange (%d)\n", newValueAsSInt32);
+    debugIOLog ("-AppleUSBAudioMuteControl::performValueChange (%d)", newValueAsSInt32);
 
 Exit:
     return kIOReturnSuccess;
@@ -144,7 +150,7 @@ Exit:
     return error;
 
 DeviceInactive:
-	debugIOLog("AppleUSBAudioMuteControl::SetCurMute ERROR attempt to send a device request to and inactive device\n");
+	debugIOLog("AppleUSBAudioMuteControl::SetCurMute ERROR attempt to send a device request to and inactive device");
 	error = kIOReturnError;
 	goto Exit;
 }
@@ -157,7 +163,7 @@ void AppleUSBAudioMuteControl::updateUSBValue (SInt32 newValue) {
     UInt8						theValue;
     IOReturn					ret;
 
-    debug2IOLog ("+AppleUSBAudioMuteControl::updateUSBValue (%d)\n", newValue);
+    debugIOLog ("+AppleUSBAudioMuteControl::updateUSBValue (%d)", newValue);
 
     theValue = (newValue != 0);
 	ret = SetCurMute (interfaceNumber, channelNumber, theValue);
@@ -169,7 +175,7 @@ void AppleUSBAudioMuteControl::updateUSBValue (SInt32 newValue) {
 		IORegistryEntry *		parent;
 		IORegistryEntry *		engine;
 
-		debugIOLog ("Looking for iSub mute control\n");
+		debugIOLog ("Looking for iSub mute control");
 		start = getParentEntry (gIOServicePlane);
 		FailIf (NULL == start, Exit);
 		parent = start->getParentEntry (gIOServicePlane);
@@ -181,7 +187,7 @@ void AppleUSBAudioMuteControl::updateUSBValue (SInt32 newValue) {
 		engine->release ();
 
 		if (NULL != iSubMute) {
-			debugIOLog ("Setting the iSub mute control\n");
+			debugIOLog ("Setting the iSub mute control");
 			iSubMute->setValue (newValue);
 			iSubMute->release ();
 		}
@@ -189,17 +195,17 @@ void AppleUSBAudioMuteControl::updateUSBValue (SInt32 newValue) {
 #endif
 //Exit:
     if (ret != kIOReturnSuccess) {
-        debug3IOLog ("++AppleUSBAudioMuteContol:updateUSBValue () - set current value for channel %d failed: 0x%x\n", channelNumber, ret);
+        debugIOLog ("++AppleUSBAudioMuteContol:updateUSBValue () - set current value for channel %d failed: 0x%x", channelNumber, ret);
     }
 
-    debug2IOLog ("-AppleUSBAudioMuteControl::updateUSBValue (%d)\n", newValue);
+    debugIOLog ("-AppleUSBAudioMuteControl::updateUSBValue (%d)", newValue);
 }
 
 void AppleUSBAudioMuteControl::updateValueCallback (void *arg1, void *arg2) {
     AppleUSBAudioMuteControl 	*muteControl;
     UInt32						value;
 
-    debug3IOLog ("+AppleUSBAudioMuteControl::updateValueCallback (%d, %d)\n", (UInt32*)arg1, (UInt32*)arg2);
+    debugIOLog ("+AppleUSBAudioMuteControl::updateValueCallback (%d, %d)", (UInt32*)arg1, (UInt32*)arg2);
     muteControl = (AppleUSBAudioMuteControl *)arg1;
     value = (UInt32)arg2;
 
@@ -207,7 +213,7 @@ void AppleUSBAudioMuteControl::updateValueCallback (void *arg1, void *arg2) {
         muteControl->updateUSBValue (value);
     }
 
-    debug3IOLog ("-AppleUSBAudioMuteControl::updateValueCallback (%d, %d)\n", (UInt32*)arg1, (UInt32*)arg2);
+    debugIOLog ("-AppleUSBAudioMuteControl::updateValueCallback (%d, %d)", (UInt32*)arg1, (UInt32*)arg2);
 }
 
 #if 0
