@@ -217,34 +217,49 @@ UInt8 USBAudioConfigObject::FindAltInterfaceWithSettings (UInt8 interfaceNum, UI
 	potentialAltInterface = 1;
 	theAltInterface = 255;
 	found = FALSE;
-	while (!found && potentialAltInterface != 255) {
-		if ((GetFormat (interfaceNum, potentialAltInterface) & 0x0FFF) != 0) {	// Make sure it's not an undefined format
+	while (!found && potentialAltInterface != 255) 
+	{
+		if ((GetFormat (interfaceNum, potentialAltInterface) & 0x0FFF) != 0) 
+		{	// Make sure it's not an undefined format
 			potentialAltInterface = FindNextAltInterfaceWithNumChannels (interfaceNum, potentialAltInterface, numChannels);
-			FailIf (255 == potentialAltInterface, Exit);
-			if (potentialAltInterface == FindNextAltInterfaceWithSampleSize (interfaceNum, potentialAltInterface, sampleSize)) {
-				FailIf (255 == potentialAltInterface, Exit);
-				if (0 != sampleRate) {
+			if (255 == potentialAltInterface)
+			{
+				debugIOLog ("Undefined format! No alternate setting ID for interface %d, %d channels", interfaceNum, numChannels);
+				break;		// Done, so break out of while loop
+			}
+			if (potentialAltInterface == FindNextAltInterfaceWithSampleSize (interfaceNum, potentialAltInterface, sampleSize)) 
+			{
+				if (0 != sampleRate) 
+				{
 					// they want a specific sample rate on this interface
-					if (potentialAltInterface == FindNextAltInterfaceWithSampleRate (interfaceNum, potentialAltInterface, sampleRate)) {
+					if (potentialAltInterface == FindNextAltInterfaceWithSampleRate (interfaceNum, potentialAltInterface, sampleRate)) 
+					{
 						found = TRUE;
 						theAltInterface = potentialAltInterface;
-					} else {
+					} 
+					else 
+					{
 						potentialAltInterface++;
 					}
-				} else {
+				} 
+				else 
+				{
 					// they don't care about the sample rate
 					found = TRUE;
 					theAltInterface = potentialAltInterface;
 				}
-			} else {
+			} 
+			else 
+			{
 				potentialAltInterface++;
 			}
-		} else {
+		} 
+		else 
+		{
 			potentialAltInterface++;
 		}
-	}
-
-Exit:
+	} // while (!found && potentialAltInterface != 255)
+	
 	return theAltInterface;
 }
 
@@ -1076,54 +1091,58 @@ void USBAudioConfigObject::ParseConfigurationDescriptor (void) {
         if (INTERFACE == ((ACInterfaceDescriptorPtr)theInterfacePtr)->bDescriptorType) {
 			debugIOLog ("in INTERFACE in ParseConfigurationDescriptor");
 			thisInterfaceNumber = ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber;
-			theInterfacePtr = ParseInterfaceDescriptor (theInterfacePtr, &interfaceClass, &interfaceSubClass);
-
-			debugIOLog ("theControlInterfaceNum = %d, thisInterfaceNumber = %d", theControlInterfaceNum, thisInterfaceNumber);
-			if (AUDIOCONTROL == interfaceSubClass && theControlInterfaceNum == thisInterfaceNumber) {
-				debugIOLog ("found a AUDIOCONTROL CS_INTERFACE ");
-				if (NULL != theControls) {
-					theControlObject = OSDynamicCast (USBAudioControlObject, theControls->getLastObject ());
-				}
-				FailIf (NULL == theControlObject, Exit);
-				theInterfacePtr = theControlObject->ParseACInterfaceDescriptor (theInterfacePtr, ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber);
-				GetControlledStreamNumbers (&streamInterfaceNumbers, &numStreamInterfaces);
-				haveControlInterface = TRUE;
-			} else if (haveControlInterface && AUDIOSTREAMING == interfaceSubClass) {
-				debugIOLog ("in CS_INTERFACE in ParseConfigurationDescriptor");
-				for (index = 0; index < numStreamInterfaces; index++) {
-					debugIOLog ("comparing thisInterfaceNum = %d with %d", thisInterfaceNumber, streamInterfaceNumbers[index]);
-					if (thisInterfaceNumber == streamInterfaceNumbers[index]) {
-						debugIOLog ("found a AUDIOSTREAMING CS_INTERFACE ");
-						if (NULL != theStreams) {
-							theStreamObject = OSDynamicCast (USBAudioStreamObject, theStreams->getLastObject ());
+			if (AUDIO == ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceClass) {
+				theInterfacePtr = ParseInterfaceDescriptor (theInterfacePtr, &interfaceClass, &interfaceSubClass);
+				debugIOLog ("theControlInterfaceNum = %d, thisInterfaceNumber = %d", theControlInterfaceNum, thisInterfaceNumber);
+				if (AUDIOCONTROL == interfaceSubClass && theControlInterfaceNum == thisInterfaceNumber) {
+					debugIOLog ("found a AUDIOCONTROL CS_INTERFACE ");
+					if (NULL != theControls) {
+						theControlObject = OSDynamicCast (USBAudioControlObject, theControls->getLastObject ());
+					}
+					FailIf (NULL == theControlObject, Exit);
+					theInterfacePtr = theControlObject->ParseACInterfaceDescriptor (theInterfacePtr, ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber);
+					GetControlledStreamNumbers (&streamInterfaceNumbers, &numStreamInterfaces);
+					haveControlInterface = TRUE;
+				} else if (haveControlInterface && AUDIOSTREAMING == interfaceSubClass) {
+					debugIOLog ("in CS_INTERFACE in ParseConfigurationDescriptor");
+					for (index = 0; index < numStreamInterfaces; index++) {
+						debugIOLog ("comparing thisInterfaceNum = %d with %d", thisInterfaceNumber, streamInterfaceNumbers[index]);
+						if (thisInterfaceNumber == streamInterfaceNumbers[index]) {
+							debugIOLog ("found a AUDIOSTREAMING CS_INTERFACE ");
+							if (NULL != theStreams) {
+								theStreamObject = OSDynamicCast (USBAudioStreamObject, theStreams->getLastObject ());
+							}
+							FailIf (NULL == theStreamObject, Exit);
+							theInterfacePtr = theStreamObject->ParseASInterfaceDescriptor (theInterfacePtr, ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber);
+							foundStreamInterface = TRUE;
+							break;			// Get out of for loop
 						}
-						FailIf (NULL == theStreamObject, Exit);
-						theInterfacePtr = theStreamObject->ParseASInterfaceDescriptor (theInterfacePtr, ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber);
-						foundStreamInterface = TRUE;
-						break;			// Get out of for loop
 					}
-				}
-				if (thisInterfaceNumber != lastInterfaceNumber) {
-					lastInterfaceNumber = thisInterfaceNumber;
-					numParsedInterfaces++;
-					if (numParsedInterfaces > numStreamInterfaces) {
-						break;		// Get out of while loop, we've parsed everything associated with this control interface
+					if (thisInterfaceNumber != lastInterfaceNumber) {
+						lastInterfaceNumber = thisInterfaceNumber;
+						numParsedInterfaces++;
+						if (numParsedInterfaces > numStreamInterfaces) {
+							break;		// Get out of while loop, we've parsed everything associated with this control interface
+						}
 					}
-				}
-			} else if (MIDISTREAMING == interfaceSubClass) {
-				debugIOLog ("MIDI, jumping forward %d bytes", theInterfacePtr->bLength);
-				for (index = 0; index < numStreamInterfaces; index++) {
-					if (thisInterfaceNumber == streamInterfaceNumbers[index]) {
-						break;
+				} else if (MIDISTREAMING == interfaceSubClass) {
+					debugIOLog ("MIDI, jumping forward %d bytes", theInterfacePtr->bLength);
+					for (index = 0; index < numStreamInterfaces; index++) {
+						if (thisInterfaceNumber == streamInterfaceNumbers[index]) {
+							break;
+						}
 					}
+					theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
+				} else if (AUDIOCONTROL == interfaceSubClass) {
+					debugIOLog ("Found a control interface that we don't care about");
+					debugIOLog ("jumping forward %d bytes", ((((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[1] << 8) | (((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[0])));
+					theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + ((((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[1] << 8) | (((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[0])));
+				} else {
+					debugIOLog ("Unknown, jumping forward %d bytes", theInterfacePtr->bLength);
+					theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
 				}
-				theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
-			} else if (AUDIOCONTROL == interfaceSubClass) {
-				debugIOLog ("Found a control interface that we don't care about");
-				debugIOLog ("jumping forward %d bytes", ((((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[1] << 8) | (((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[0])));
-				theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + ((((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[1] << 8) | (((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[0])));
 			} else {
-				debugIOLog ("Unknown, jumping forward %d bytes", theInterfacePtr->bLength);
+				debugIOLog ("not an audio interface in ParseConfigurationDescriptor, jumping forward %d bytes", theInterfacePtr->bLength);
 				theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
 			}
 		} else {
